#!/usr/bin/env python3
import csv, psycopg2

class UtilCursor(psycopg2.extensions.cursor):

    def execute_file(self, filename):
        with open(filename) as f:
            self.execute(f.read())
    
    def measure_once(self, operation, parameters=None):
        # psycopg2 automagically translates the json output
        self.execute("EXPLAIN (ANALYZE, FORMAT JSON) " + operation, parameters)
        return self.fetchone()[0][0]["Total Runtime"]
        
    def measure(self, operation, parameters=None, iterations=3):
        return sum(self.measure_once(operation, parameters) 
                    for i in range(iterations)) / iterations;

def measure_query(cur, query, date=None):
    if date:
        report = "{}({})".format(query, date)
        query = "EXECUTE {}(%s)".format(query)
        return (report, cur.measure(query, (date,)))
    else:
        report = "{}".format(query, date)
        query = "EXECUTE {}".format(query)
        return (report, cur.measure(query))
        
def report(result, log = csv.writer(open('results.csv', 'w'))):
    print("{}: {:.7} ms".format(*result))
    log.writerow(result)
        
       
# connect to db
conn = psycopg2.connect(dbname='is_proj1')
cur = conn.cursor(cursor_factory=UtilCursor)

# get parameter for dates, see http://dba.stackexchange.com/questions/17086
query = """
    SELECT bucket, max({attribute})
    FROM (
        SELECT {attribute}, ntile(5) over (order by {attribute}) as bucket
        FROM {table}
    ) AS tmp
    GROUP BY bucket ORDER BY bucket
"""

print("Calculating query parameteres...")

cur.execute(query.format(attribute='l_shipdate', table='lineitem'))
shipdates = [pair[1] for pair in cur.fetchall()]

cur.execute(query.format(attribute='o_orderdate', table='orders'))
orderdates = [pair[1] for pair in cur.fetchall()] 

cur.execute_file("row_queries.sql")
cur.execute_file("sc_queries.sql")
cur.execute_file("sc_idx_queries.sql")

print("Row-oriented Baseline")

for query in ['Q1', 'Q2', 'Q3']:
    for date in shipdates: 
        result = measure_query(cur, query, date)
        report(result)
        
for query in ['Q4', 'Q5']:
    for date in orderdates: 
        result = measure_query(cur, query, date)
        report(result)

for query in ['Q7']:
    result = measure_query(cur, query)
    report(result)

print("Simple Column Schema")

for query in ['SC_Q1', 'SC_Q2', 'SC_Q3']:
    for date in shipdates: 
        result = measure_query(cur, query, date)
        report(result)
        
for query in ['SC_Q4', 'SC_Q5']:
    for date in orderdates: 
        result = measure_query(cur, query, date)
        report(result)

for query in ['SC_Q7']:
    result = measure_query(cur, query)
    report(result)
    
print("Indexed Simple Column Schema")

for query in ['SCIDX_Q1', 'SCIDX_Q2', 'SCIDX_Q3']:
    for date in shipdates: 
        result = measure_query(cur, query, date)
        report(result)
        
for query in ['SCIDX_Q4', 'SCIDX_Q5']:
    for date in orderdates: 
        result = measure_query(cur, query, date)
        report(result)

for query in ['SCIDX_Q7']:
    result = measure_query(cur, query)
    report(result)

#print(cur.measure("EXECUTE Q7"));
